/*
 * glade-intro.css
 *
 * Copyright (C) 2017 Juan Pablo Ugarte
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Author: Juan Pablo Ugarte <juanpablougarte@gmail.com>
 * 
 */

/* GladeWindow */
@binding-set WindowBindings {
  bind "<Control>s"         { "glade-action" ("save") };
  bind "<Control>p"         { "glade-action" ("preferences") };
  bind "<Control>w"         { "glade-action" ("close") };
  bind "<Control>z"         { "glade-action" ("undo") };
  bind "<Control><shift>z"  { "glade-action" ("redo") };
  bind "<Control>x"         { "glade-action" ("cut") };
  bind "<Control>c"         { "glade-action" ("copy") };
  bind "<Control>v"         { "glade-action" ("paste") };
  bind "Delete"             { "glade-action" ("delete") };
  bind "<Control>Page_Up"   { "glade-action" ("previous") };
  bind "<Control>Page_Down" { "glade-action" ("next") };
  bind "<Control>n"         { "glade-action" ("new") };
  bind "<Control>o"         { "glade-action" ("open") };
  bind "<Control>q"         { "glade-action" ("quit") };
  bind "F1"                 { "glade-action" ("reference") };
  bind "<Alt>1"             { "glade-switch" (0) };
  bind "<Alt>2"             { "glade-switch" (1) };
  bind "<Alt>3"             { "glade-switch" (2) };
  bind "<Alt>4"             { "glade-switch" (3) };
  bind "<Alt>5"             { "glade-switch" (4) };
  bind "<Alt>6"             { "glade-switch" (5) };
  bind "<Alt>7"             { "glade-switch" (6) };
  bind "<Alt>8"             { "glade-switch" (7) };
  bind "<Alt>9"             { "glade-switch" (8) };
  bind "<Alt>0"             { "glade-switch" (9) };
}

GladeWindow {
  -gtk-key-bindings: WindowBindings;
}

.glade-tight-fit {
  margin: 0;
  padding: 0;
}

@keyframes brand-highlight {
  from { -gtk-icon-shadow: 2px -1px 4px alpha(@warning_color, 0); }
  to   { -gtk-icon-shadow: 2px -1px 4px alpha(@warning_color, 0.8); }
}

#glade-brand-image {
  animation: brand-highlight 2s infinite alternate;
}

#glade-brand-image:backdrop {
  animation: none;
}

#glade-label {
  font-size: 64px;
  font-weight: bold;
  text-shadow: 2px 2px 2px gray;
}

#version-label {
  font-size: 18px;
  font-weight: bold;
  text-shadow: 1px 1px 2px gray;
  padding-bottom: 120px;
}

/* GladeIntro */

popover.glade-intro {
  padding: 1em;
}

popover.glade-intro label {
  font-size: 18px;
  font-weight: bold;
}

@keyframes widget-highlight {
  from { box-shadow: 0px 0px 10px @theme_selected_bg_color, inset 0px 0px 4px  @theme_selected_bg_color; }
  to   { box-shadow: 0px 0px 4px  @theme_selected_bg_color, inset 0px 0px 10px @theme_selected_bg_color; }
}

button.glade-intro-highlight,
buttonbox.glade-intro-highlight > button,
stackswitcher.glade-intro-highlight > button,
#adaptor-chooser.glade-intro-highlight button,
#inspector.glade-intro-highlight > box > scrolledwindow > treeview,
#editor.glade-intro-highlight > notebook > stack > scrolledwindow {
  animation: widget-highlight 1s infinite alternate;
}

/* Quartz workarounds to improve performance (issue #442) */

/* Disable window shadow */
GladeWindow.Quartz > decoration {
  box-shadow: none;
}

/* Disable Glade logo shadow animation */
GladeWindow.Quartz #glade-brand-image {
  animation: none;
}

/* Add window border since there is no shadow */
GladeWindow.Quartz {
  border: 1px solid lightgray;
  border-top: unset;
}

