#ifndef __GLADE_PREVIEW_TOKENS_H__
#define __GLADE_PREVIEW_TOKENS_H__

#define UPDATE_TOKEN "<update>\n"
#define UPDATE_TOKEN_SIZE strlen (UPDATE_TOKEN)

#define QUIT_TOKEN "<quit>\n"
#define QUIT_TOKEN_SIZE strlen (QUIT_TOKEN)

#endif /* __GLADE_PREVIEW_TOKENS_H__ */

