/*
 * glade-design-layout.css
 *
 * Copyright (C) 2016 Juan Pablo Ugarte
 * 
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Author: Juan Pablo Ugarte <juanpablougarte@gmail.com>
 * 
 */

glade-design-layout {
  background: none;
  border: 4px solid @borders;
  color: @theme_selected_fg_color;
  border-radius: 4px 4px 0px 4px;
}

glade-design-layout:selected {
  border: 4px solid @theme_selected_bg_color;
}

glade-design-layout.handle {
  background-color: @borders;
  border-radius: 0px 0px 4px 4px;
}

glade-design-layout:selected.handle {
  background-color: @theme_selected_bg_color;
  border-radius: 0px 0px 4px 4px;
}

glade-design-layout.selection {
  border: 2px solid @theme_selected_bg_color;
  border-radius: 0px;
  background-color:transparent;
  background-image: -gtk-gradient (radial,
                                   center center, 0,
                                   center center, 1,
                                   from (alpha(@theme_selected_bg_color, .08)),
                                   to   (alpha(@theme_selected_bg_color, .50)));
}

