/*
 * glade.h
 *
 * Copyright (C) 2007 The GNOME Foundation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef __GLADE_H__
#define __GLADE_H__

#include <gladeui/glade-xml-utils.h>
#include <gladeui/glade-widget-adaptor.h>
#include <gladeui/glade-widget.h>
#include <gladeui/glade-property-def.h>
#include <gladeui/glade-property-shell.h>
#include <gladeui/glade-property.h>
#include <gladeui/glade-signal-def.h>
#include <gladeui/glade-signal-model.h>
#include <gladeui/glade-signal.h>
#include <gladeui/glade-project.h>
#include <gladeui/glade-app.h>
#include <gladeui/glade-base-editor.h>
#include <gladeui/glade-command.h>
#include <gladeui/glade-design-view.h>
#include <gladeui/glade-editor.h>
#include <gladeui/glade-editor-skeleton.h>
#include <gladeui/glade-editor-property.h>
#include <gladeui/glade-palette.h>
#include <gladeui/glade-clipboard.h>
#include <gladeui/glade-inspector.h>
#include <gladeui/glade-placeholder.h>
#include <gladeui/glade-utils.h>
#include <gladeui/glade-builtins.h>
#include <gladeui/glade-name-context.h>
#include <gladeui/glade-editable.h>
#include <gladeui/glade-displayable-values.h>
#include <gladeui/glade-cell-renderer-icon.h>
#include <gladeui/glade-cursor.h>
#include <gladeui/glade-debug.h>

#endif /* __GLADE_H__ */
