DESCRIPTION "Implements the core glade functions needed by plugins"
EXPORTS
   glade_catalog_load_all
   glade_command_create
   glade_cursor_init
   glade_editor_load_widget
   glade_placeholder_get_type
   glade_placeholder_new
   glade_project_get_tooltips
   glade_project_new
   glade_project_open
   glade_project_remove_widget
   glade_project_selection_get
   glade_project_window_get
   glade_project_window_get_active_project
   glade_project_window_new
   glade_project_window_new_project
   glade_project_window_open_project
   glade_project_window_show_all
   glade_property_set
   glade_setup_log_handlers
   glade_widget_class_dump_param_specs
   glade_widget_class_get_by_name
   glade_widget_get_project
   glade_widget_get_property
   glade_widget_get_type
   glade_widget_new_for_internal_child
